
#pragma once

#include <memory>
#include <winhttp.h>
#include <boost/exception/all.hpp>
#include <boost/optional.hpp>

namespace WinHTTPWrapper {

bool	InitWinHTTP(boost::optional<CString> optUserAgent = boost::none, boost::optional<CString> optProxy = boost::none);
void	TermWinHTTP();

struct HINTERNET_deleter
{
	typedef HINTERNET	pointer;
	void operator () (HINTERNET handle) {
		ATLVERIFY(::WinHttpCloseHandle(handle));
	}
};

typedef std::unique_ptr<HINTERNET, HINTERNET_deleter>	INetHandle;


typedef boost::error_info<struct err_info, DWORD> ex_add_info;

struct WinHTTPException : virtual boost::exception, virtual std::exception
{
	WinHTTPException() 
	{
		dwGetLastError = ::GetLastError();
		*this << ex_add_info(dwGetLastError);
	}

	DWORD	dwGetLastError;
};

class CUrl
{
public:
	CUrl() { }

	CUrl(const CString& url) : m_url(url)
	{
		Set(url);
	}

	CUrl(const std::wstring& url) : m_url(url.c_str())
	{
		Set(url.c_str());
	}

	void	Set(const CString& url)
	{
		URL_COMPONENTS urlComp = { sizeof(URL_COMPONENTS) };
		urlComp.dwHostNameLength	= -1;
		urlComp.dwUrlPathLength		= -1;
		if (::WinHttpCrackUrl(url, url.GetLength(), 0, &urlComp) == FALSE) 
			BOOST_THROW_EXCEPTION(WinHTTPException());
		m_hostName	= CString(urlComp.lpszHostName, urlComp.dwHostNameLength);
		m_path		= CString(urlComp.lpszUrlPath, urlComp.dwUrlPathLength);
		if (urlComp.nScheme == INTERNET_SCHEME_HTTPS)
			m_optSSLPort = urlComp.nPort;
	}

	const CString&	GetURL() const { return m_url; }
	const CString&	GetHost() const { return m_hostName; }
	const CString&	GetPath() const { return m_path; }
	boost::optional<int>	GetSSLPortNumber() const { return m_optSSLPort; }

private:
	CString	m_url;
	CString	m_hostName;
	CString	m_path;
	boost::optional<int>	m_optSSLPort;
};

/// urlŎw肵T[o[Ƃ̃RlNV쐬
INetHandle	HttpConnect(const CUrl& url);

/// RlNVۂ̃\[Xɑ΂郊NGXg쐬
INetHandle	HttpOpenRequest(const CUrl& url, const INetHandle& hConnect, LPCWSTR Verb = L"GET", const CString& referer = CString());

/// NGXgwb_ǉ
void		HttpAddRequestHeaders(const INetHandle& hRequest, const CString& addHeaders, DWORD dwModifiers = WINHTTP_ADDREQ_FLAG_ADD);

/// NGXg̐ݒύX
void		HttpSetOption(const INetHandle& hRequest, DWORD option, DWORD optionValue);

/// NGXgɃvNVݒ肷
void		HttpSetProxy(const INetHandle& hRequest, const CString& proxy);

/// NGXg𑗐MA܂ő҂
bool		HttpSendRequestAndReceiveResponse(const INetHandle& hRequest, const std::string& postData = std::string());

/// NGXgʂ̃Xe[^XR[hԂ
DWORD		HttpQueryStatusCode(const INetHandle& hRequest);

/// InfoLevelŎw肵X|Xwb_Ԃ
bool		HttpQueryHeaders(const INetHandle& hRequest, DWORD InfoLevel, CString& headerContents);

/// InfoLevelŎw肵X|Xwb_Ԃ
bool		HttpQueryHeaders(const INetHandle& hRequest, DWORD InfoLevel, DWORD& headerContents);

// X|Xwb_𐶂̂܂ܕԂ
bool		HttpQueryRawHeaders(const INetHandle& hRequest, CString& rawHeaderContents);

/// X|X{fB擾
std::string HttpReadData(const INetHandle& hRequest);

/// url_E[h
boost::optional<std::string>	HttpDownloadData(const CString& url);


}	// namespace WinHTTPWrapper